<?php

class MLumen_Descriptor_Single_Post {

	/**
	 * @var Mlumen_Options
	 */
	private $api_options;

	/**
	 * @var Mlumen_Api_Descriptor
	 */
	private $api_descriptor;

	/**
	 * @var array
	 */
	public $register_taxonomy;

	public function __construct( Mlumen_Options $api_options, Mlumen_Api_Descriptor $api_descriptor ) {
		$this->api_options    = $api_options;
		$this->api_descriptor = $api_descriptor;

		$this->register_taxonomy = $this->api_options->get_taxonomy_prompts();
	}

	/**
	 * Gutenberg init
	 */
	public function gutenberg_init() {
		if ( current_user_can( 'edit_posts' ) ) {
			wp_register_script(
				'mlumen-gutenberg-tags',
				MLUMEN_PLUGIN_URL . 'admin/js/gutenberg-tags.min.js',
				array('lodash', 'wp-api-fetch', 'wp-data', 'wp-element', 'wp-i18n', 'wp-url')
			);

			register_block_type( 'mlumen/tags', array(
				'api_version'   => 2,
				'editor_script' => 'mlumen-gutenberg-tags',
			) );
		}
	}

	/**
	 * Register mLumen meta box
	 */
	public function register_meta_box() {
		add_filter( 'register_taxonomy_args', function ( $args, $taxonomy ) {

			// Replace the original metabox callback with our wrapper
			if ( in_array( $taxonomy, $this->register_taxonomy, true ) || substr( $taxonomy, - 7 ) === '_mlumen' ) {
				$args['meta_box_cb'] = array( $this, 'mlumen_meta_box' );
			}

			return $args;
		}, 10, 2 );
	}

	/**
	 * Get tag list
	 *
	 * @return array|false
	 */
	function get_tags( string $text ) {
		if ( $this->api_descriptor->get_suggest( $text ) ) {
			return array_map( function ( $item ) {
				return $item['name'];
			}, $this->api_descriptor->data );
		}

		return false;
	}

	/**
	 * Add tags when post is saving
	 */
	public function post_auto_save( int $post_id, WP_Post $post ) {
		// Autosaving, fail.
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}

		if ( $this->api_options->options['taxonomy_prompts_type'] == 3 ) {

			$fields = $this->api_options->get_taxonomy_for_post_save( $post->post_type );

			if ( count( $fields ) > 0 ) {
				$tags_names = $this->get_tags( $post->post_content );
				if ( $tags_names !== false ) {
					foreach ( $fields as $field ) {
						wp_set_post_terms( $post_id, $tags_names, $field, $this->api_options->options['taxonomy_append'] );
					}
				}
			}
		}
	}

	/**
	 * Add property mLumen for meta box
	 */
	function mlumen_meta_box( WP_Post $post, array $box ) {
		$taxonomy = $box['args']['taxonomy'];

		post_tags_meta_box( $post, $box );

		if ( $this->api_options->options['taxonomy_prompts_type'] === 1 ):
			?>
            <p>
				<?php echo __( 'Proponowane przez e-usługę', 'mlumen' ); ?>
                <b> <?php echo __( 'DESKRYPTOR', 'mlumen' ); ?></b>
                <a href="#" class="descriptor_single_action" data-taxonomy="<?php echo esc_attr( $taxonomy ); ?>">
					<?php echo __( 'odśwież', 'mlumen' ); ?>
                </a>
            </p>
            <div id="descriptor_single_<?php echo esc_attr( $taxonomy ); ?>" class="the-tagcloud" style="display:none">
                <ul class="wp-tag-cloud" role="list"></ul>
            </div>
		<?php endif; ?>

		<?php if ( $this->api_options->options['taxonomy_prompts_type'] === 2 ):
			?>
            <p>
                <a href="#" class="descriptor_auto_action" data-taxonomy="<?php echo esc_attr( $taxonomy ); ?>">
					<?php echo __( 'Dodaj tagi proponowane przez e-usługę', 'mlumen' ); ?>
                    <b> <?php echo __( 'DESKRYPTOR', 'mlumen' ); ?></b>
                </a>
            </p>
		<?php endif; ?>

        <div id="descriptor_single_<?php echo esc_attr( $taxonomy ); ?>_message"></div>

		<?php
	}

	/**
	 * Register taxonomy mLumen
	 */
	public function register_taxonomy_mlumen() {

		$args = array(
			'labels'            => array(
				'name'          => __( 'Tagi DBN', 'mlumen' ),
				'singular_name' => __( 'Tagi DBN', 'mlumen' ),
				'menu_name'     => __( 'Tagi DBN', 'mlumen' )
			),
			'hierarchical'      => false,
			'public'            => true,
			'show_ui'           => true,
			'show_admin_column' => true,
			'show_in_nav_menus' => true,
			'show_tagcloud'     => true,
			'show_in_rest'      => true,
			'rewrite'           => array(
				'slug'         => 'mlumen',
				'with_front'   => true,
				'hierarchical' => true
			),
		);

		foreach ( $this->api_options->options['post_create_taxonomy'] as $post_type ) {
			$register_post_type = $post_type . '_mlumen';
			register_taxonomy( $register_post_type, array( $post_type ), $args );
		}
	}

}